==> 0.0.4.tar.gz: lib/dynamic_tabs.dart <==
import 'dart:io';

import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter_platform_widgets/flutter_platform_widgets.dart';
import 'package:shared_preferences/shared_preferences.dart';

import 'data/classes/tab.dart';
import 'ui/more_screen.dart';

export 'package:dynamic_tabs/data/classes/tab.dart';

class DynamicTabScaffold extends StatefulWidget {
  DynamicTabScaffold({
    @required this.tabs,
    this.backgroundColor,
    this.persistIndex = false,
    this.iconSize,
    this.maxTabs = 4,
    // this.fixedColor,
    this.type,
    // this.unselectedItemColor,
  })  : adaptive = false,
        assert(tabs != null),
        assert(tabs.length >= 2);

  DynamicTabScaffold.adaptive({
    @required this.tabs,
    this.backgroundColor,
    this.persistIndex = false,
    this.maxTabs = 4,
  })  : adaptive = true,
        type = null,
        // fixedColor = null,
        iconSize = null,
        // unselectedItemColor = null,
        assert(tabs != null),
        assert(tabs.length >= 2);

  final List<DynamicTab> tabs;
  final bool adaptive;
  final Color backgroundColor;
  final bool persistIndex;
  final int maxTabs;

  // Material Only
  final double iconSize;
  // final Color fixedColor;
  final BottomNavigationBarType type;
  // final Color unselectedItemColor;

  @override
  _DynamicTabScaffoldState createState() => _DynamicTabScaffoldState();
}

class _DynamicTabScaffoldState extends State<DynamicTabScaffold> {
  int _currentIndex = 0;

  SharedPreferences _prefs;

  List<DynamicTab> _items;

  @override
  void initState() {
    _items = widget.tabs;
    SharedPreferences.getInstance().then((value) {
      _prefs = value;
      _loadSavedTabs();
      if (widget.persistIndex) _loadIndex();
    });
    super.initState();
  }

  void _loadSavedTabs() {
    List<String> _tabs = _prefs.getStringList("bottom_tabs");
    if (_tabs != null && _tabs.isNotEmpty) {
      List<DynamicTab> _newOrder = [];
      for (var item in _tabs) {
        _newOrder.add(_items.firstWhere((t) => t.tag == item));
      }
      setState(() {
        _items = _newOrder;
      });
    } else {
      _saveNewTabs();
    }
  }

  void _saveNewTabs() {
    _prefs.setStringList("bottom_tabs", _items.map((t) => t.tag).toList());
  }

  void _loadIndex() {
    int _index = _prefs.getInt("nav_index");
    if (_index > widget.maxTabs) {
      _index = 0;
      _saveIndex();
    }
    if (_index != null) {
      setState(() {
        _currentIndex = _index;
      });
    } else {
      _saveIndex();
    }
  }

  void _saveIndex() {
    _prefs.setInt("nav_index", _currentIndex);
  }

  @override
  Widget build(BuildContext context) {
    if (widget.adaptive) {
      return PlatformScaffold(
        body: _getBody(context),
        bottomNavBar: PlatformNavBar(
          items: _showEditTab
              ? (_items.take(widget.maxTabs).toList().map((t) => t.tab).toList()
                ..add(BottomNavigationBarItem(
                  title: Text("More"),
                  icon: Icon(Icons.more_horiz),
                )))
              : _items.map((t) => t.tab).toList(),
          currentIndex: _currentIndex,
          itemChanged: _tabChanged,
          backgroundColor: widget?.backgroundColor,
        ),
      );
    }

    return Scaffold(
      body: _getBody(context),
      bottomNavigationBar: BottomNavigationBar(
        items: _showEditTab
            ? (_items.take(widget.maxTabs).toList().map((t) => t.tab).toList()
              ..add(BottomNavigationBarItem(
                title: Text("More"),
                icon: Icon(Icons.more_horiz),
              )))
            : _items.map((t) => t.tab).toList(),
        currentIndex: _currentIndex,
        onTap: _tabChanged,
        // backgroundColor: widget?.backgroundColor,
        fixedColor: widget?.backgroundColor ?? Theme.of(context).primaryColor,
        type: widget?.type,
        // unselectedItemColor: widget?.unselectedItemColor ?? Colors.grey,
      ),
    );
  }

  void _tabChanged(int index) {
    print("Index: $index");
    setState(() {
      _currentIndex = index;
    });
    if (widget.persistIndex) _saveIndex();
  }

  Widget _getBody(BuildContext context) {
    if (_showEditTab && _moreTab) {
      if (widget.adaptive && Platform.isIOS) {
        return CupertinoTabView(
          builder: (BuildContext context) => MoreTab(
                maxTabs: widget.maxTabs,
                adaptive: widget.adaptive,
                tabs: _items,
                tabsChanged: (List<DynamicTab> tabs) {
                  setState(() {
                    _items = tabs;
                  });
                  _saveNewTabs();
                },
              ),
        );
      }
      return MoreTab(
        maxTabs: widget.maxTabs,
        adaptive: widget.adaptive,
        tabs: _items,
        tabsChanged: (List<DynamicTab> tabs) {
          setState(() {
            _items = tabs;
          });
          _saveNewTabs();
        },
      );
    }

    if (widget.adaptive && Platform.isIOS)
      return CupertinoTabView(
        builder: (BuildContext context) => CupertinoPageScaffold(
              navigationBar: CupertinoNavigationBar(
                middle: _items[_currentIndex]?.title ??
                    _items[_currentIndex]?.tab?.title,
                trailing: _items[_currentIndex]?.trailingAction,
              ),
              child: _items[_currentIndex].child,
            ),
      );

    return Scaffold(
      appBar: AppBar(
        title:
            _items[_currentIndex]?.title ?? _items[_currentIndex]?.tab?.title,
        actions: <Widget>[_items[_currentIndex]?.trailingAction ?? Container()],
      ),
      body: _items[_currentIndex].child,
    );
  }

  bool get _moreTab => _currentIndex == widget.maxTabs;

  bool get _showEditTab => _items.length > widget.maxTabs;
}

==> 0.0.6.tar.gz: lib/dynamic_tabs.dart <==
import 'dart:io';

import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:shared_preferences/shared_preferences.dart';

import 'data/classes/tab.dart';
import 'ui/more_screen.dart';
export 'data/classes/tab.dart';

class DynamicTabScaffold extends StatefulWidget {
  DynamicTabScaffold({
    @required this.tabs,
    this.backgroundColor,
    this.persistIndex = false,
    this.iconSize,
    this.maxTabs = 4,
    this.type = BottomNavigationBarType.fixed,
  })  : adaptive = false,
        routes = null,
        assert(tabs != null),
        assert(tabs.length >= 2);

  DynamicTabScaffold.adaptive({
    @required this.tabs,
    this.backgroundColor,
    this.persistIndex = false,
    this.maxTabs = 4,
    @required this.routes,
  })  : adaptive = true,
        type = null,
        iconSize = null,
        assert(tabs != null),
        assert(tabs.length >= 2);

  final List<DynamicTab> tabs;
  final bool adaptive;
  final Color backgroundColor;
  final bool persistIndex;
  final int maxTabs;
  final Map<String, WidgetBuilder> routes;

  // Material Only
  final double iconSize;
  // final Color fixedColor;
  final BottomNavigationBarType type;
  // final Color unselectedItemColor;

  @override
  _DynamicTabScaffoldState createState() => _DynamicTabScaffoldState();
}

class _DynamicTabScaffoldState extends State<DynamicTabScaffold> {
  int _currentIndex = 0;

  SharedPreferences _prefs;

  List<DynamicTab> _items;

  @override
  void initState() {
    _items = widget.tabs;
    SharedPreferences.getInstance().then((value) {
      _prefs = value;
      _loadSavedTabs();
      if (widget.persistIndex) _loadIndex();
    });
    super.initState();
  }

  void _loadSavedTabs() {
    List<String> _tabs = _prefs.getStringList("bottom_tabs");
    if (_tabs != null && _tabs.isNotEmpty) {
      List<DynamicTab> _newOrder = [];
      for (var item in _tabs) {
        _newOrder.add(_items.firstWhere((t) => t.tag == item));
      }
      setState(() {
        _items = _newOrder;
      });
    } else {
      _saveNewTabs();
    }
  }

  void _saveNewTabs() {
    _prefs.setStringList("bottom_tabs", _items.map((t) => t.tag).toList());
  }

  void _loadIndex() {
    int _index = _prefs.getInt("nav_index");
    if (_index > widget.maxTabs) {
      _index = 0;
      _saveIndex();
    }
    if (_index != null) {
      setState(() {
        _currentIndex = _index;
      });
    } else {
      _saveIndex();
    }
  }

  void _saveIndex() {
    _prefs.setInt("nav_index", _currentIndex);
  }

  @override
  Widget build(BuildContext context) {
    if (widget.adaptive && Platform.isIOS) {
      return CupertinoTabScaffold(
        tabBuilder: (BuildContext context, int index) {
          return _getBody(context);
        },
        tabBar: CupertinoTabBar(
          items: _showEditTab
              ? (_items.take(widget.maxTabs).toList().map((t) => t.tab).toList()
                ..add(BottomNavigationBarItem(
                  title: Text("More"),
                  icon: Icon(Icons.more_horiz),
                )))
              : _items.map((t) => t.tab).toList(),
          currentIndex: _currentIndex,
          onTap: _tabChanged,
          backgroundColor: widget?.backgroundColor,
        ),
      );
    }

    return Scaffold(
      body: _getBody(context),
      bottomNavigationBar: BottomNavigationBar(
        items: _showEditTab
            ? (_items.take(widget.maxTabs).toList().map((t) => t.tab).toList()
              ..add(BottomNavigationBarItem(
                title: Text("More"),
                icon: Icon(Icons.more_horiz),
              )))
            : _items.map((t) => t.tab).toList(),
        currentIndex: _currentIndex,
        onTap: _tabChanged,

        fixedColor: widget?.backgroundColor ?? Theme.of(context).primaryColor,
        type: widget?.type,
        // unselectedItemColor: widget?.unselectedItemColor ?? Colors.grey,
      ),
    );
  }

  void _tabChanged(int index) {
    print("Index: $index");
    setState(() {
      _currentIndex = index;
    });
    if (widget.persistIndex) _saveIndex();
  }

  Widget _getBody(BuildContext context) {
    if (_showEditTab && _moreTab) {
      if (widget.adaptive && Platform.isIOS) {
        return CupertinoTabView(
          routes: widget.routes,
          builder: (BuildContext context) => MoreTab(
                maxTabs: widget.maxTabs,
                adaptive: widget.adaptive,
                tabs: _items,
                tabsChanged: (List<DynamicTab> tabs) {
                  setState(() {
                    _items = tabs;
                  });
                  _saveNewTabs();
                },
              ),
        );
      }
      return MoreTab(
        maxTabs: widget.maxTabs,
        adaptive: widget.adaptive,
        tabs: _items,
        tabsChanged: (List<DynamicTab> tabs) {
          setState(() {
            _items = tabs;
          });
          _saveNewTabs();
        },
      );
    }

    if (widget.adaptive && Platform.isIOS) {
      return CupertinoTabView(
        routes: widget.routes,
        builder: (BuildContext context) => _items[_currentIndex].child,
      );
    }

    return _items[_currentIndex].child;
  }

  bool get _moreTab => _currentIndex == widget.maxTabs;

  bool get _showEditTab => _items.length > widget.maxTabs;
}
